/*
* $LastChangedBy$
* $LastChangedDate$
* $LastChangedRevision$
* $HeadURL$
*
* Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
*/

import Foundation

class BlueConicCookieStorage: HTTPCookieStorage {
    
    private var bcCookies = [String: HTTPCookie]()
    
    override init() {
        super.init()
        self.loadCookies()
    }
    
    /** Adds the cookies. */
    func addCookies(_ cookies: [HTTPCookie]) {
        cookies.forEach { cookie in
            bcCookies[cookie.name] = cookie;
        }
        self.saveCookies()
    }

    /** Returns the cookies. */
    func getCookies() -> [HTTPCookie] {
        let date = Date()
        
        return bcCookies.values.filter { c in
            if let expire = c.expiresDate  {
                return expire >= date
            }
            return false;
        }
    }

    /** Saves the cookies. */
    internal func saveCookies() {
        let array = self.getCookies().compactMap { (cookie) -> [HTTPCookiePropertyKey: Any]? in
            cookie.properties
        }
        UserDefaults.standard.set(array, forKey: "bc_cookies")
        UserDefaults.standard.synchronize()
    }
    
    /** Loads the cookies. */
    internal func loadCookies() {
        guard let storedCookies = UserDefaults.standard.value(forKey: "bc_cookies") as? [[HTTPCookiePropertyKey: Any]] else {
            return
        }
        
        var cookies = [HTTPCookie]()
        storedCookies.forEach { (cookieProps) in
            let cookie = HTTPCookie(properties: cookieProps)
            cookies.append(cookie!)
        }
        self.addCookies(cookies)
    }
}
