/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */
import Foundation

private struct BlueConicQueue {
    static let queue = DispatchQueue(label: Constants.QUEUE_LABEL)
}

extension BlueConicClient {
    /**
     Add the closure to the blueconic queue, which will run all closures in the queue synchronously.
     By default use this function. Only call synchronized(queue: closure:) if you want to add the closure to another queue
     */
    @objc class func synchronized(_ closure: @escaping () -> Void) {
        synchronized(BlueConicQueue.queue, closure: closure)
    }

    /**
     Add the closure to a given background queue, which will run all closures in the queue synchronously.
     */
    @objc class func synchronized(_ queue: DispatchQueue, closure: @escaping () -> Void) {
        let currentQueueLabel = String(cString: __dispatch_queue_get_label(nil), encoding: .utf8)
        if currentQueueLabel == queue.label {
            closure()
        } else {
            queue.sync(execute: closure)
        }
    }
}
